@@ -3,29 +3,29 @@
 
 // Hello world example for browserify.
 
-var pdfjsLib = require("pdfjs-dist");
+const pdfjsLib = require("pdfjs-dist");
 
-var pdfPath = "../learning/helloworld.pdf";
+const pdfPath = "../learning/helloworld.pdf";
 
 // Setting worker path to worker bundle.
 pdfjsLib.GlobalWorkerOptions.workerSrc =
   "../../build/browserify/pdf.worker.bundle.js";
 
 // Loading a document.
-var loadingTask = pdfjsLib.getDocument(pdfPath);
+const loadingTask = pdfjsLib.getDocument(pdfPath);
 loadingTask.promise
   .then(function (pdfDocument) {
     // Request a first page
     return pdfDocument.getPage(1).then(function (pdfPage) {
       // Display page on the existing canvas with 100% scale.
-      var viewport = pdfPage.getViewport({ scale: 1.0 });
-      var canvas = document.getElementById("theCanvas");
+      const viewport = pdfPage.getViewport({ scale: 1.0 });
+      const canvas = document.getElementById("theCanvas");
       canvas.width = viewport.width;
       canvas.height = viewport.height;
-      var ctx = canvas.getContext("2d");
-      var renderTask = pdfPage.render({
+      const ctx = canvas.getContext("2d");
+      const renderTask = pdfPage.render({
         canvasContext: ctx,
-        viewport: viewport,
+        viewport,
       });
       return renderTask.promise;
     });
