@@ -2,14 +2,16 @@
 
 // Small subset of the webL10n API by Fabien Cazenave for PDF.js extension.
 (function (window) {
-  var gLanguage = "";
-  var gExternalLocalizerServices = null;
-  var gReadyState = "loading";
+  let gL10nData = null;
+  let gLanguage = "";
+  let gExternalLocalizerServices = null;
+  let gReadyState = "loading";
 
   // fetch an l10n objects
   function getL10nData(key) {
-    var response = gExternalLocalizerServices.getStrings(key);
-    var data = JSON.parse(response);
+    gL10nData ||= gExternalLocalizerServices.getStrings();
+
+    const data = gL10nData?.[key];
     if (!data) {
       console.warn("[l10n] #" + key + " missing for [" + gLanguage + "]");
     }
