@@ -16,16 +16,16 @@
 import {
   CMapCompressionType,
   FormatError,
-  isString,
   unreachable,
   warn,
 } from "../shared/util.js";
-import { isCmd, isEOF, isName, isStream } from "./primitives.js";
+import { Cmd, EOF, isCmd, Name } from "./primitives.js";
+import { BaseStream } from "./base_stream.js";
 import { Lexer } from "./parser.js";
 import { MissingDataException } from "./core_utils.js";
 import { Stream } from "./stream.js";
 
-var BUILT_IN_CMAPS = [
+const BUILT_IN_CMAPS = [
   // << Start unicode maps.
   "Adobe-GB1-UCS2",
   "Adobe-CNS1-UCS2",
@@ -239,13 +239,20 @@ class CMap {
     if (high - low > MAX_MAP_RANGE) {
       throw new Error("mapBfRange - ignoring data above MAX_MAP_RANGE.");
     }
-    var lastByte = dstLow.length - 1;
+    const lastByte = dstLow.length - 1;
     while (low <= high) {
       this._map[low++] = dstLow;
-      // Only the last byte has to be incremented.
+      // Only the last byte has to be incremented (in the normal case).
+      const nextCharCode = dstLow.charCodeAt(lastByte) + 1;
+      if (nextCharCode > 0xff) {
+        dstLow =
+          dstLow.substring(0, lastByte - 1) +
+          String.fromCharCode(dstLow.charCodeAt(lastByte - 1) + 1) +
+          "\x00";
+        continue;
+      }
       dstLow =
-        dstLow.substring(0, lastByte) +
-        String.fromCharCode(dstLow.charCodeAt(lastByte) + 1);
+        dstLow.substring(0, lastByte) + String.fromCharCode(nextCharCode);
     }
   }
 
@@ -337,6 +344,22 @@ class CMap {
     out.length = 1;
   }
 
+  getCharCodeLength(charCode) {
+    const codespaceRanges = this.codespaceRanges;
+    for (let n = 0, nn = codespaceRanges.length; n < nn; n++) {
+      // Check each codespace range to see if it falls within.
+      const codespaceRange = codespaceRanges[n];
+      for (let k = 0, kk = codespaceRange.length; k < kk; ) {
+        const low = codespaceRange[k++];
+        const high = codespaceRange[k++];
+        if (charCode >= low && charCode <= high) {
+          return n + 1;
+        }
+      }
+    }
+    return 1;
+  }
+
   get length() {
     return this._map.length;
   }
@@ -420,10 +443,10 @@ class IdentityCMap extends CMap {
   }
 }
 
-var BinaryCMapReader = (function BinaryCMapReaderClosure() {
+const BinaryCMapReader = (function BinaryCMapReaderClosure() {
   function hexToInt(a, size) {
-    var n = 0;
-    for (var i = 0; i <= size; i++) {
+    let n = 0;
+    for (let i = 0; i <= size; i++) {
       n = (n << 8) | a[i];
     }
     return n >>> 0;
@@ -442,8 +465,8 @@ var BinaryCMapReader = (function BinaryCMapReaderClosure() {
   }
 
   function addHex(a, b, size) {
-    var c = 0;
-    for (var i = size; i >= 0; i--) {
+    let c = 0;
+    for (let i = size; i >= 0; i--) {
       c += a[i] + b[i];
       a[i] = c & 255;
       c >>= 8;
@@ -451,36 +474,37 @@ var BinaryCMapReader = (function BinaryCMapReaderClosure() {
   }
 
   function incHex(a, size) {
-    var c = 1;
-    for (var i = size; i >= 0 && c > 0; i--) {
+    let c = 1;
+    for (let i = size; i >= 0 && c > 0; i--) {
       c += a[i];
       a[i] = c & 255;
       c >>= 8;
     }
   }
 
-  var MAX_NUM_SIZE = 16;
-  var MAX_ENCODED_NUM_SIZE = 19; // ceil(MAX_NUM_SIZE * 7 / 8)
+  const MAX_NUM_SIZE = 16;
+  const MAX_ENCODED_NUM_SIZE = 19; // ceil(MAX_NUM_SIZE * 7 / 8)
 
-  function BinaryCMapStream(data) {
-    this.buffer = data;
-    this.pos = 0;
-    this.end = data.length;
-    this.tmpBuf = new Uint8Array(MAX_ENCODED_NUM_SIZE);
-  }
+  class BinaryCMapStream {
+    constructor(data) {
+      this.buffer = data;
+      this.pos = 0;
+      this.end = data.length;
+      this.tmpBuf = new Uint8Array(MAX_ENCODED_NUM_SIZE);
+    }
 
-  BinaryCMapStream.prototype = {
     readByte() {
       if (this.pos >= this.end) {
         return -1;
       }
       return this.buffer[this.pos++];
-    },
+    }
+
     readNumber() {
-      var n = 0;
-      var last;
+      let n = 0;
+      let last;
       do {
-        var b = this.readByte();
+        const b = this.readByte();
         if (b < 0) {
           throw new FormatError("unexpected EOF in bcmap");
         }
@@ -488,33 +512,36 @@ var BinaryCMapReader = (function BinaryCMapReaderClosure() {
         n = (n << 7) | (b & 0x7f);
       } while (!last);
       return n;
-    },
+    }
+
     readSigned() {
-      var n = this.readNumber();
+      const n = this.readNumber();
       return n & 1 ? ~(n >>> 1) : n >>> 1;
-    },
+    }
+
     readHex(num, size) {
       num.set(this.buffer.subarray(this.pos, this.pos + size + 1));
       this.pos += size + 1;
-    },
+    }
+
     readHexNumber(num, size) {
-      var last;
-      var stack = this.tmpBuf,
-        sp = 0;
+      let last;
+      const stack = this.tmpBuf;
+      let sp = 0;
       do {
-        var b = this.readByte();
+        const b = this.readByte();
         if (b < 0) {
           throw new FormatError("unexpected EOF in bcmap");
         }
         last = !(b & 0x80);
         stack[sp++] = b & 0x7f;
       } while (!last);
-      var i = size,
+      let i = size,
         buffer = 0,
         bufferSize = 0;
       while (i >= 0) {
         while (bufferSize < 8 && stack.length > 0) {
-          buffer = (stack[--sp] << bufferSize) | buffer;
+          buffer |= stack[--sp] << bufferSize;
           bufferSize += 7;
         }
         num[i] = buffer & 255;
@@ -522,43 +549,46 @@ var BinaryCMapReader = (function BinaryCMapReaderClosure() {
         buffer >>= 8;
         bufferSize -= 8;
       }
-    },
+    }
+
     readHexSigned(num, size) {
       this.readHexNumber(num, size);
-      var sign = num[size] & 1 ? 255 : 0;
-      var c = 0;
-      for (var i = 0; i <= size; i++) {
+      const sign = num[size] & 1 ? 255 : 0;
+      let c = 0;
+      for (let i = 0; i <= size; i++) {
         c = ((c & 1) << 8) | num[i];
         num[i] = (c >> 1) ^ sign;
       }
-    },
+    }
+
     readString() {
-      var len = this.readNumber();
-      var s = "";
-      for (var i = 0; i < len; i++) {
+      const len = this.readNumber();
+      let s = "";
+      for (let i = 0; i < len; i++) {
         s += String.fromCharCode(this.readNumber());
       }
       return s;
-    },
-  };
+    }
+  }
 
-  function processBinaryCMap(data, cMap, extend) {
-    return new Promise(function (resolve, reject) {
-      var stream = new BinaryCMapStream(data);
-      var header = stream.readByte();
+  // eslint-disable-next-line no-shadow
+  class BinaryCMapReader {
+    async process(data, cMap, extend) {
+      const stream = new BinaryCMapStream(data);
+      const header = stream.readByte();
       cMap.vertical = !!(header & 1);
 
-      var useCMap = null;
-      var start = new Uint8Array(MAX_NUM_SIZE);
-      var end = new Uint8Array(MAX_NUM_SIZE);
-      var char = new Uint8Array(MAX_NUM_SIZE);
-      var charCode = new Uint8Array(MAX_NUM_SIZE);
-      var tmp = new Uint8Array(MAX_NUM_SIZE);
-      var code;
+      let useCMap = null;
+      const start = new Uint8Array(MAX_NUM_SIZE);
+      const end = new Uint8Array(MAX_NUM_SIZE);
+      const char = new Uint8Array(MAX_NUM_SIZE);
+      const charCode = new Uint8Array(MAX_NUM_SIZE);
+      const tmp = new Uint8Array(MAX_NUM_SIZE);
+      let code;
 
-      var b;
+      let b;
       while ((b = stream.readByte()) >= 0) {
-        var type = b >> 5;
+        const type = b >> 5;
         if (type === 7) {
           // metadata, e.g. comment or usecmap
           switch (b & 0x1f) {
@@ -571,16 +601,15 @@ var BinaryCMapReader = (function BinaryCMapReaderClosure() {
           }
           continue;
         }
-        var sequence = !!(b & 0x10);
-        var dataSize = b & 15;
+        const sequence = !!(b & 0x10);
+        const dataSize = b & 15;
 
         if (dataSize + 1 > MAX_NUM_SIZE) {
-          throw new Error("processBinaryCMap: Invalid dataSize.");
+          throw new Error("BinaryCMapReader.process: Invalid dataSize.");
         }
 
-        var ucs2DataSize = 1;
-        var subitemsCount = stream.readNumber();
-        var i;
+        const ucs2DataSize = 1;
+        const subitemsCount = stream.readNumber();
         switch (type) {
           case 0: // codespacerange
             stream.readHex(start, dataSize);
@@ -591,7 +620,7 @@ var BinaryCMapReader = (function BinaryCMapReaderClosure() {
               hexToInt(start, dataSize),
               hexToInt(end, dataSize)
             );
-            for (i = 1; i < subitemsCount; i++) {
+            for (let i = 1; i < subitemsCount; i++) {
               incHex(end, dataSize);
               stream.readHexNumber(start, dataSize);
               addHex(start, end, dataSize);
@@ -610,7 +639,7 @@ var BinaryCMapReader = (function BinaryCMapReaderClosure() {
             addHex(end, start, dataSize);
             stream.readNumber(); // code
             // undefined range, skipping
-            for (i = 1; i < subitemsCount; i++) {
+            for (let i = 1; i < subitemsCount; i++) {
               incHex(end, dataSize);
               stream.readHexNumber(start, dataSize);
               addHex(start, end, dataSize);
@@ -624,7 +653,7 @@ var BinaryCMapReader = (function BinaryCMapReaderClosure() {
             stream.readHex(char, dataSize);
             code = stream.readNumber();
             cMap.mapOne(hexToInt(char, dataSize), code);
-            for (i = 1; i < subitemsCount; i++) {
+            for (let i = 1; i < subitemsCount; i++) {
               incHex(char, dataSize);
               if (!sequence) {
                 stream.readHexNumber(tmp, dataSize);
@@ -644,7 +673,7 @@ var BinaryCMapReader = (function BinaryCMapReaderClosure() {
               hexToInt(end, dataSize),
               code
             );
-            for (i = 1; i < subitemsCount; i++) {
+            for (let i = 1; i < subitemsCount; i++) {
               incHex(end, dataSize);
               if (!sequence) {
                 stream.readHexNumber(start, dataSize);
@@ -669,7 +698,7 @@ var BinaryCMapReader = (function BinaryCMapReaderClosure() {
               hexToInt(char, ucs2DataSize),
               hexToStr(charCode, dataSize)
             );
-            for (i = 1; i < subitemsCount; i++) {
+            for (let i = 1; i < subitemsCount; i++) {
               incHex(char, ucs2DataSize);
               if (!sequence) {
                 stream.readHexNumber(tmp, ucs2DataSize);
@@ -694,7 +723,7 @@ var BinaryCMapReader = (function BinaryCMapReaderClosure() {
               hexToInt(end, ucs2DataSize),
               hexToStr(charCode, dataSize)
             );
-            for (i = 1; i < subitemsCount; i++) {
+            for (let i = 1; i < subitemsCount; i++) {
               incHex(end, ucs2DataSize);
               if (!sequence) {
                 stream.readHexNumber(start, ucs2DataSize);
@@ -713,40 +742,31 @@ var BinaryCMapReader = (function BinaryCMapReaderClosure() {
             }
             break;
           default:
-            reject(new Error("processBinaryCMap: Unknown type: " + type));
-            return;
+            throw new Error(`BinaryCMapReader.process - unknown type: ${type}`);
         }
       }
 
       if (useCMap) {
-        resolve(extend(useCMap));
-        return;
+        return extend(useCMap);
       }
-      resolve(cMap);
-    });
+      return cMap;
+    }
   }
 
-  // eslint-disable-next-line no-shadow
-  function BinaryCMapReader() {}
-
-  BinaryCMapReader.prototype = {
-    process: processBinaryCMap,
-  };
-
   return BinaryCMapReader;
 })();
 
-var CMapFactory = (function CMapFactoryClosure() {
+const CMapFactory = (function CMapFactoryClosure() {
   function strToInt(str) {
-    var a = 0;
-    for (var i = 0; i < str.length; i++) {
+    let a = 0;
+    for (let i = 0; i < str.length; i++) {
       a = (a << 8) | str.charCodeAt(i);
     }
     return a >>> 0;
   }
 
   function expectString(obj) {
-    if (!isString(obj)) {
+    if (typeof obj !== "string") {
       throw new FormatError("Malformed CMap: expected string.");
     }
   }
@@ -759,45 +779,45 @@ var CMapFactory = (function CMapFactoryClosure() {
 
   function parseBfChar(cMap, lexer) {
     while (true) {
-      var obj = lexer.getObj();
-      if (isEOF(obj)) {
+      let obj = lexer.getObj();
+      if (obj === EOF) {
         break;
       }
       if (isCmd(obj, "endbfchar")) {
         return;
       }
       expectString(obj);
-      var src = strToInt(obj);
+      const src = strToInt(obj);
       obj = lexer.getObj();
       // TODO are /dstName used?
       expectString(obj);
-      var dst = obj;
+      const dst = obj;
       cMap.mapOne(src, dst);
     }
   }
 
   function parseBfRange(cMap, lexer) {
     while (true) {
-      var obj = lexer.getObj();
-      if (isEOF(obj)) {
+      let obj = lexer.getObj();
+      if (obj === EOF) {
         break;
       }
       if (isCmd(obj, "endbfrange")) {
         return;
       }
       expectString(obj);
-      var low = strToInt(obj);
+      const low = strToInt(obj);
       obj = lexer.getObj();
       expectString(obj);
-      var high = strToInt(obj);
+      const high = strToInt(obj);
       obj = lexer.getObj();
-      if (Number.isInteger(obj) || isString(obj)) {
-        var dstLow = Number.isInteger(obj) ? String.fromCharCode(obj) : obj;
+      if (Number.isInteger(obj) || typeof obj === "string") {
+        const dstLow = Number.isInteger(obj) ? String.fromCharCode(obj) : obj;
         cMap.mapBfRange(low, high, dstLow);
       } else if (isCmd(obj, "[")) {
         obj = lexer.getObj();
-        var array = [];
-        while (!isCmd(obj, "]") && !isEOF(obj)) {
+        const array = [];
+        while (!isCmd(obj, "]") && obj !== EOF) {
           array.push(obj);
           obj = lexer.getObj();
         }
@@ -811,101 +831,100 @@ var CMapFactory = (function CMapFactoryClosure() {
 
   function parseCidChar(cMap, lexer) {
     while (true) {
-      var obj = lexer.getObj();
-      if (isEOF(obj)) {
+      let obj = lexer.getObj();
+      if (obj === EOF) {
         break;
       }
       if (isCmd(obj, "endcidchar")) {
         return;
       }
       expectString(obj);
-      var src = strToInt(obj);
+      const src = strToInt(obj);
       obj = lexer.getObj();
       expectInt(obj);
-      var dst = obj;
+      const dst = obj;
       cMap.mapOne(src, dst);
     }
   }
 
   function parseCidRange(cMap, lexer) {
     while (true) {
-      var obj = lexer.getObj();
-      if (isEOF(obj)) {
+      let obj = lexer.getObj();
+      if (obj === EOF) {
         break;
       }
       if (isCmd(obj, "endcidrange")) {
         return;
       }
       expectString(obj);
-      var low = strToInt(obj);
+      const low = strToInt(obj);
       obj = lexer.getObj();
       expectString(obj);
-      var high = strToInt(obj);
+      const high = strToInt(obj);
       obj = lexer.getObj();
       expectInt(obj);
-      var dstLow = obj;
+      const dstLow = obj;
       cMap.mapCidRange(low, high, dstLow);
     }
   }
 
   function parseCodespaceRange(cMap, lexer) {
     while (true) {
-      var obj = lexer.getObj();
-      if (isEOF(obj)) {
+      let obj = lexer.getObj();
+      if (obj === EOF) {
         break;
       }
       if (isCmd(obj, "endcodespacerange")) {
         return;
       }
-      if (!isString(obj)) {
+      if (typeof obj !== "string") {
         break;
       }
-      var low = strToInt(obj);
+      const low = strToInt(obj);
       obj = lexer.getObj();
-      if (!isString(obj)) {
+      if (typeof obj !== "string") {
         break;
       }
-      var high = strToInt(obj);
+      const high = strToInt(obj);
       cMap.addCodespaceRange(obj.length, low, high);
     }
     throw new FormatError("Invalid codespace range.");
   }
 
   function parseWMode(cMap, lexer) {
-    var obj = lexer.getObj();
+    const obj = lexer.getObj();
     if (Number.isInteger(obj)) {
       cMap.vertical = !!obj;
     }
   }
 
   function parseCMapName(cMap, lexer) {
-    var obj = lexer.getObj();
-    if (isName(obj) && isString(obj.name)) {
+    const obj = lexer.getObj();
+    if (obj instanceof Name) {
       cMap.name = obj.name;
     }
   }
 
-  function parseCMap(cMap, lexer, fetchBuiltInCMap, useCMap) {
-    var previous;
-    var embeddedUseCMap;
+  async function parseCMap(cMap, lexer, fetchBuiltInCMap, useCMap) {
+    let previous, embeddedUseCMap;
     objLoop: while (true) {
       try {
-        var obj = lexer.getObj();
-        if (isEOF(obj)) {
+        const obj = lexer.getObj();
+        if (obj === EOF) {
           break;
-        } else if (isName(obj)) {
+        } else if (obj instanceof Name) {
           if (obj.name === "WMode") {
             parseWMode(cMap, lexer);
           } else if (obj.name === "CMapName") {
             parseCMapName(cMap, lexer);
           }
           previous = obj;
-        } else if (isCmd(obj)) {
+        } else if (obj instanceof Cmd) {
           switch (obj.cmd) {
             case "endcmap":
               break objLoop;
             case "usecmap":
-              if (isName(previous)) {
+              if (previous instanceof Name) {
                 embeddedUseCMap = previous.name;
               }
               break;
@@ -943,97 +962,85 @@ var CMapFactory = (function CMapFactoryClosure() {
     if (useCMap) {
       return extendCMap(cMap, fetchBuiltInCMap, useCMap);
     }
-    return Promise.resolve(cMap);
+    return cMap;
   }
 
-  function extendCMap(cMap, fetchBuiltInCMap, useCMap) {
-    return createBuiltInCMap(useCMap, fetchBuiltInCMap).then(function (
-      newCMap
-    ) {
-      cMap.useCMap = newCMap;
-      // If there aren't any code space ranges defined clone all the parent ones
-      // into this cMap.
-      if (cMap.numCodespaceRanges === 0) {
-        var useCodespaceRanges = cMap.useCMap.codespaceRanges;
-        for (var i = 0; i < useCodespaceRanges.length; i++) {
-          cMap.codespaceRanges[i] = useCodespaceRanges[i].slice();
-        }
-        cMap.numCodespaceRanges = cMap.useCMap.numCodespaceRanges;
+  async function extendCMap(cMap, fetchBuiltInCMap, useCMap) {
+    cMap.useCMap = await createBuiltInCMap(useCMap, fetchBuiltInCMap);
+    // If there aren't any code space ranges defined clone all the parent ones
+    // into this cMap.
+    if (cMap.numCodespaceRanges === 0) {
+      const useCodespaceRanges = cMap.useCMap.codespaceRanges;
+      for (let i = 0; i < useCodespaceRanges.length; i++) {
+        cMap.codespaceRanges[i] = useCodespaceRanges[i].slice();
+      }
+      cMap.numCodespaceRanges = cMap.useCMap.numCodespaceRanges;
+    }
+    // Merge the map into the current one, making sure not to override
+    // any previously defined entries.
+    cMap.useCMap.forEach(function (key, value) {
+      if (!cMap.contains(key)) {
+        cMap.mapOne(key, cMap.useCMap.lookup(key));
       }
-      // Merge the map into the current one, making sure not to override
-      // any previously defined entries.
-      cMap.useCMap.forEach(function (key, value) {
-        if (!cMap.contains(key)) {
-          cMap.mapOne(key, cMap.useCMap.lookup(key));
-        }
-      });
-
-      return cMap;
     });
+
+    return cMap;
   }
 
-  function createBuiltInCMap(name, fetchBuiltInCMap) {
+  async function createBuiltInCMap(name, fetchBuiltInCMap) {
     if (name === "Identity-H") {
-      return Promise.resolve(new IdentityCMap(false, 2));
+      return new IdentityCMap(false, 2);
     } else if (name === "Identity-V") {
-      return Promise.resolve(new IdentityCMap(true, 2));
+      return new IdentityCMap(true, 2);
     }
     if (!BUILT_IN_CMAPS.includes(name)) {
-      return Promise.reject(new Error("Unknown CMap name: " + name));
+      throw new Error("Unknown CMap name: " + name);
     }
     if (!fetchBuiltInCMap) {
-      return Promise.reject(
-        new Error("Built-in CMap parameters are not provided.")
-      );
+      throw new Error("Built-in CMap parameters are not provided.");
     }
 
-    return fetchBuiltInCMap(name).then(function (data) {
-      var cMapData = data.cMapData,
-        compressionType = data.compressionType;
-      var cMap = new CMap(true);
+    const { cMapData, compressionType } = await fetchBuiltInCMap(name);
+    const cMap = new CMap(true);
 
-      if (compressionType === CMapCompressionType.BINARY) {
-        return new BinaryCMapReader().process(cMapData, cMap, function (
-          useCMap
-        ) {
-          return extendCMap(cMap, fetchBuiltInCMap, useCMap);
-        });
-      }
-      if (compressionType === CMapCompressionType.NONE) {
-        var lexer = new Lexer(new Stream(cMapData));
-        return parseCMap(cMap, lexer, fetchBuiltInCMap, null);
-      }
-      return Promise.reject(
-        new Error(
-          "TODO: Only BINARY/NONE CMap compression is currently supported."
-        )
-      );
-    });
+    if (compressionType === CMapCompressionType.BINARY) {
+      return new BinaryCMapReader().process(cMapData, cMap, useCMap => {
+        return extendCMap(cMap, fetchBuiltInCMap, useCMap);
+      });
+    }
+    if (compressionType === CMapCompressionType.NONE) {
+      const lexer = new Lexer(new Stream(cMapData));
+      return parseCMap(cMap, lexer, fetchBuiltInCMap, null);
+    }
+    throw new Error(
+      "TODO: Only BINARY/NONE CMap compression is currently supported."
+    );
   }
 
   return {
     async create(params) {
-      var encoding = params.encoding;
-      var fetchBuiltInCMap = params.fetchBuiltInCMap;
-      var useCMap = params.useCMap;
+      const encoding = params.encoding;
+      const fetchBuiltInCMap = params.fetchBuiltInCMap;
+      const useCMap = params.useCMap;
 
-      if (isName(encoding)) {
+      if (encoding instanceof Name) {
         return createBuiltInCMap(encoding.name, fetchBuiltInCMap);
-      } else if (isStream(encoding)) {
-        var cMap = new CMap();
-        var lexer = new Lexer(encoding);
-        return parseCMap(cMap, lexer, fetchBuiltInCMap, useCMap).then(function (
-          parsedCMap
-        ) {
-          if (parsedCMap.isIdentityCMap) {
-            return createBuiltInCMap(parsedCMap.name, fetchBuiltInCMap);
-          }
-          return parsedCMap;
-        });
+      } else if (encoding instanceof BaseStream) {
+        const parsedCMap = await parseCMap(
+          /* cMap = */ new CMap(),
+          /* lexer = */ new Lexer(encoding),
+          fetchBuiltInCMap,
+          useCMap
+        );
+
+        if (parsedCMap.isIdentityCMap) {
+          return createBuiltInCMap(parsedCMap.name, fetchBuiltInCMap);
+        }
+        return parsedCMap;
       }
       throw new Error("Encoding required.");
     },
   };
 })();
 
-export { CMap, IdentityCMap, CMapFactory };
+export { CMap, CMapFactory, IdentityCMap };
