@@ -13,7 +13,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-/* eslint-disable object-shorthand */
+/* eslint-disable no-var */
 
 "use strict";
 
@@ -24,90 +24,169 @@ var os = require("os");
 var puppeteer = require("puppeteer");
 var url = require("url");
 var testUtils = require("./testutils.js");
+const dns = require("dns");
+const readline = require("readline");
+const yargs = require("yargs");
+
+// Chrome uses host `127.0.0.1` in the browser's websocket endpoint URL while
+// Firefox uses `localhost`, which before Node.js 17 also resolved to the IPv4
+// address `127.0.0.1` by Node.js' DNS resolver. However, this behavior changed
+// in Node.js 17 where the default is to prefer an IPv6 address if one is
+// offered (which varies based on the OS and/or how the `localhost` hostname
+// resolution is configured), so it can now also resolve to `::1`. This causes
+// Firefox to not start anymore since it doesn't bind on the `::1` interface.
+// To avoid this, we switch Node.js' DNS resolver back to preferring IPv4
+// since we connect to a local browser anyway. Only do this for Node.js versions
+// that actually have this API since it got introduced in Node.js 14.18.0 and
+// it's not relevant for older versions anyway.
+if (dns.setDefaultResultOrder !== undefined) {
+  dns.setDefaultResultOrder("ipv4first");
+}
 
 function parseOptions() {
-  function describeCheck(fn, text) {
-    fn.toString = function () {
-      return text;
-    };
-    return fn;
-  }
-
-  var yargs = require("yargs")
+  yargs
     .usage("Usage: $0")
-    .boolean([
-      "help",
-      "masterMode",
-      "reftest",
-      "unitTest",
-      "fontTest",
-      "noPrompts",
-      "noDownload",
-      "noChrome",
-      "downloadOnly",
-      "strictVerify",
-    ])
-    .string(["manifestFile", "port", "statsFile", "statsDelay", "testfilter"])
-    .alias("help", "h")
-    .alias("masterMode", "m")
-    .alias("testfilter", "t")
-    .describe("help", "Show this help message")
-    .describe("masterMode", "Run the script in master mode.")
-    .describe(
-      "noPrompts",
-      "Uses default answers (intended for CLOUD TESTS only!)."
-    )
-    .describe(
-      "manifestFile",
-      "A path to JSON file in the form of test_manifest.json"
-    )
-    .default("manifestFile", "test_manifest.json")
-    .describe(
-      "reftest",
-      "Automatically start reftest showing comparison " +
-        "test failures, if there are any."
-    )
-    .describe("testfilter", "Run specific reftest(s).")
-    .default("testfilter", [])
+    .option("downloadOnly", {
+      default: false,
+      describe: "Download test PDFs without running the tests.",
+      type: "boolean",
+    })
+    .option("fontTest", {
+      default: false,
+      describe: "Run the font tests.",
+      type: "boolean",
+    })
+    .option("help", {
+      alias: "h",
+      default: false,
+      describe: "Show this help message.",
+      type: "boolean",
+    })
+    .option("integration", {
+      default: false,
+      describe: "Run the integration tests.",
+      type: "boolean",
+    })
+    .option("manifestFile", {
+      default: "test_manifest.json",
+      describe: "A path to JSON file in the form of `test_manifest.json`.",
+      type: "string",
+    })
+    .option("masterMode", {
+      alias: "m",
+      default: false,
+      describe: "Run the script in master mode.",
+      type: "boolean",
+    })
+    .option("noChrome", {
+      default: false,
+      describe: "Skip Chrome when running tests.",
+      type: "boolean",
+    })
+    .option("noDownload", {
+      default: false,
+      describe: "Skip downloading of test PDFs.",
+      type: "boolean",
+    })
+    .option("noPrompts", {
+      default: false,
+      describe: "Uses default answers (intended for CLOUD TESTS only!).",
+      type: "boolean",
+    })
+    .option("port", {
+      default: 0,
+      describe: "The port the HTTP server should listen on.",
+      type: "number",
+    })
+    .option("reftest", {
+      default: false,
+      describe:
+        "Automatically start reftest showing comparison test failures, if there are any.",
+      type: "boolean",
+    })
+    .option("statsDelay", {
+      default: 0,
+      describe:
+        "The amount of time in milliseconds the browser should wait before starting stats.",
+      type: "number",
+    })
+    .option("statsFile", {
+      default: "",
+      describe: "The file where to store stats.",
+      type: "string",
+    })
+    .option("strictVerify", {
+      default: false,
+      describe: "Error if verifying the manifest files fails.",
+      type: "boolean",
+    })
+    .option("testfilter", {
+      alias: "t",
+      default: [],
+      describe: "Run specific reftest(s).",
+      type: "array",
+    })
     .example(
-      "$0 -t=issue5567 -t=issue5909",
-      "Run the reftest identified by issue5567 and issue5909."
+      "testfilter",
+      "$0 -t=issue5567 -t=issue5909\n" +
+        "Run the reftest identified by issue5567 and issue5909."
     )
-    .describe("port", "The port the HTTP server should listen on.")
-    .default("port", 0)
-    .describe("unitTest", "Run the unit tests.")
-    .describe("fontTest", "Run the font tests.")
-    .describe("noDownload", "Skips test PDFs downloading.")
-    .describe("noChrome", "Skip Chrome when running tests.")
-    .describe("downloadOnly", "Download test PDFs without running the tests.")
-    .describe("strictVerify", "Error if verifying the manifest files fails.")
-    .describe("statsFile", "The file where to store stats.")
-    .describe(
-      "statsDelay",
-      "The amount of time in milliseconds the browser " +
-        "should wait before starting stats."
-    )
-    .default("statsDelay", 0)
-    .check(
-      describeCheck(function (argv) {
-        return (
-          +argv.reftest + argv.unitTest + argv.fontTest + argv.masterMode <= 1
-        );
-      }, "--reftest, --unitTest, --fontTest and --masterMode must not be " +
-        "specified at the same time.")
-    )
-    .check(
-      describeCheck(function (argv) {
-        return !argv.noDownload || !argv.downloadOnly;
-      }, "--noDownload and --downloadOnly cannot be used together.")
-    )
-    .check(
-      describeCheck(function (argv) {
-        return !argv.masterMode || argv.manifestFile === "test_manifest.json";
-      }, "when --masterMode is specified --manifestFile shall be equal " +
-        "test_manifest.json")
-    );
-  var result = yargs.argv;
+    .option("unitTest", {
+      default: false,
+      describe: "Run the unit tests.",
+      type: "boolean",
+    })
+    .option("xfaOnly", {
+      default: false,
+      describe: "Only run the XFA reftest(s).",
+      type: "boolean",
+    })
+    .check(argv => {
+      if (
+        +argv.reftest + argv.unitTest + argv.fontTest + argv.masterMode <=
+        1
+      ) {
+        return true;
+      }
+      throw new Error(
+        "--reftest, --unitTest, --fontTest, and --masterMode must not be specified together."
+      );
+    })
+    .check(argv => {
+      if (
+        +argv.unitTest + argv.fontTest + argv.integration + argv.xfaOnly <=
+        1
+      ) {
+        return true;
+      }
+      throw new Error(
+        "--unitTest, --fontTest, --integration, and --xfaOnly must not be specified together."
+      );
+    })
+    .check(argv => {
+      if (argv.testfilter && argv.testfilter.length > 0 && argv.xfaOnly) {
+        throw new Error("--testfilter and --xfaOnly cannot be used together.");
+      }
+      return true;
+    })
+    .check(argv => {
+      if (!argv.noDownload || !argv.downloadOnly) {
+        return true;
+      }
+      throw new Error(
+        "--noDownload and --downloadOnly cannot be used together."
+      );
+    })
+    .check(argv => {
+      if (!argv.masterMode || argv.manifestFile === "test_manifest.json") {
+        return true;
+      }
+      throw new Error(
+        "when --masterMode is specified --manifestFile shall be equal to `test_manifest.json`."
+      );
+    });
+
+  const result = yargs.argv;
   if (result.help) {
     yargs.showHelp();
     process.exit(0);
@@ -151,14 +230,17 @@ function updateRefImages() {
     sync(false); // don't remove tmp/ for botio
     return;
   }
-  testUtils.confirm(
+
+  const reader = readline.createInterface(process.stdin, process.stdout);
+  reader.question(
     "Would you like to update the master copy in ref/? [yn] ",
-    function (confirmed) {
-      if (confirmed) {
+    function (answer) {
+      if (answer.toLowerCase() === "y") {
         sync(true);
       } else {
         console.log("  OK, not updating.");
       }
+      reader.close();
     }
   );
 }
@@ -227,11 +309,9 @@ function startRefTest(masterMode, showRefImages) {
     } else if (showRefImages && numEqFailures > 0) {
       console.log();
       console.log(
-        "Starting reftest harness to examine " +
-          numEqFailures +
-          " eq test failures."
+        `Starting reftest harness to examine ${numEqFailures} eq test failures.`
       );
-      examineRefImages(numEqFailures);
+      examineRefImages();
     }
   }
 
@@ -254,7 +334,7 @@ function startRefTest(masterMode, showRefImages) {
     onAllSessionsClosed = finalize;
 
     const startUrl = `http://${host}:${server.port}/test/test_slave.html`;
-    startBrowsers(startUrl, function (session) {
+    startBrowsers(function (session) {
       session.masterMode = masterMode;
       session.taskResults = {};
       session.tasks = {};
@@ -271,7 +351,7 @@ function startRefTest(masterMode, showRefImages) {
       session.numEqNoSnapshot = 0;
       session.numEqFailures = 0;
       monitorBrowserTimeout(session, handleSessionTimeout);
-    });
+    }, makeTestUrl(startUrl));
   }
   function checkRefsTmp() {
     if (masterMode && fs.existsSync(refsTmpDir)) {
@@ -282,13 +362,16 @@ function startRefTest(masterMode, showRefImages) {
       }
       console.log("Temporary snapshot dir tmp/ is still around.");
       console.log("tmp/ can be removed if it has nothing you need.");
-      testUtils.confirm(
+
+      const reader = readline.createInterface(process.stdin, process.stdout);
+      reader.question(
         "SHOULD THIS SCRIPT REMOVE tmp/? THINK CAREFULLY [yn] ",
-        function (confirmed) {
-          if (confirmed) {
+        function (answer) {
+          if (answer.toLowerCase() === "y") {
             testUtils.removeDirSync(refsTmpDir);
           }
           setup();
+          reader.close();
         }
       );
     } else {
@@ -328,14 +411,18 @@ function handleSessionTimeout(session) {
 function getTestManifest() {
   var manifest = JSON.parse(fs.readFileSync(options.manifestFile));
 
-  var testFilter = options.testfilter.slice(0);
-  if (testFilter.length) {
+  const testFilter = options.testfilter.slice(0),
+    xfaOnly = options.xfaOnly;
+  if (testFilter.length || xfaOnly) {
     manifest = manifest.filter(function (item) {
       var i = testFilter.indexOf(item.id);
       if (i !== -1) {
         testFilter.splice(i, 1);
         return true;
       }
+      if (xfaOnly && item.enableXfa) {
+        return true;
+      }
       return false;
     });
     if (testFilter.length) {
@@ -364,7 +451,8 @@ function checkEq(task, results, browser, masterMode) {
     if (!pageResults[page]) {
       continue;
     }
-    var testSnapshot = pageResults[page].snapshot;
+    const pageResult = pageResults[page];
+    let testSnapshot = pageResult.snapshot;
     if (testSnapshot && testSnapshot.startsWith("data:image/png;base64,")) {
       testSnapshot = Buffer.from(testSnapshot.substring(22), "base64");
     } else {
@@ -405,8 +493,8 @@ function checkEq(task, results, browser, masterMode) {
           refSnapshot
         );
 
-        // NB: this follows the format of Mozilla reftest output so that
-        // we can reuse its reftest-analyzer script
+        // This no longer follows the format of Mozilla reftest output.
+        const viewportString = `(${pageResult.viewportWidth}x${pageResult.viewportHeight}x${pageResult.outputScale})`;
         fs.appendFileSync(
           eqLog,
           "REFTEST TEST-UNEXPECTED-FAIL | " +
@@ -416,10 +504,10 @@ function checkEq(task, results, browser, masterMode) {
             "-page" +
             (page + 1) +
             " | image comparison (==)\n" +
-            "REFTEST   IMAGE 1 (TEST): " +
+            `REFTEST   IMAGE 1 (TEST)${viewportString}: ` +
             path.join(testSnapshotDir, page + 1 + ".png") +
             "\n" +
-            "REFTEST   IMAGE 2 (REFERENCE): " +
+            `REFTEST   IMAGE 2 (REFERENCE)${viewportString}: ` +
             path.join(testSnapshotDir, page + 1 + "_ref.png") +
             "\n"
         );
@@ -467,9 +555,14 @@ function checkFBF(task, results, browser, masterMode) {
       continue;
     }
     if (r0Page.snapshot !== r1Page.snapshot) {
-      // The FBF tests fail intermittently in Google Chrome when run on the
-      // bots, ignoring `makeref` failures for now; see https://github.com/mozilla/pdf.js/pull/11491
-      if (masterMode && /chrom(e|ium)/i.test(browser)) {
+      // The FBF tests fail intermittently in Firefox and Google Chrome when run
+      // on the bots, ignoring `makeref` failures for now; see
+      //  - https://github.com/mozilla/pdf.js/pull/12368
+      //  - https://github.com/mozilla/pdf.js/pull/11491
+      //
+      // TODO: Figure out why this happens, so that we can remove the hack; see
+      //       https://github.com/mozilla/pdf.js/issues/12371
+      if (masterMode) {
         console.log(
           "TEST-SKIPPED | forward-back-forward test " +
             task.id +
@@ -641,15 +734,18 @@ function refTestPostHandler(req, res) {
     }
 
     taskResults[round][page] = {
-      failure: failure,
-      snapshot: snapshot,
+      failure,
+      snapshot,
+      viewportWidth: data.viewportWidth,
+      viewportHeight: data.viewportHeight,
+      outputScale: data.outputScale,
     };
     if (stats) {
       stats.push({
-        browser: browser,
+        browser,
         pdf: id,
-        page: page,
-        round: round,
+        page,
+        round,
         stats: data.stats,
       });
     }
@@ -665,11 +761,9 @@ function refTestPostHandler(req, res) {
   return true;
 }
 
-function startUnitTest(testUrl, name) {
-  var startTime = Date.now();
-  startServer();
-  server.hooks.POST.push(unitTestPostHandler);
-  onAllSessionsClosed = function () {
+function onAllSessionsClosedAfterTests(name) {
+  const startTime = Date.now();
+  return function () {
     stopServer();
     var numRuns = 0,
       numErrors = 0;
@@ -688,12 +782,54 @@ function startUnitTest(testUrl, name) {
     var runtime = (Date.now() - startTime) / 1000;
     console.log(name + " tests runtime was " + runtime.toFixed(1) + " seconds");
   };
+}
+
+function makeTestUrl(startUrl) {
+  return function (browserName) {
+    const queryParameters =
+      `?browser=${encodeURIComponent(browserName)}` +
+      `&manifestFile=${encodeURIComponent("/test/" + options.manifestFile)}` +
+      `&testFilter=${JSON.stringify(options.testfilter)}` +
+      `&xfaOnly=${options.xfaOnly}` +
+      `&delay=${options.statsDelay}` +
+      `&masterMode=${options.masterMode}`;
+    return startUrl + queryParameters;
+  };
+}
+
+function startUnitTest(testUrl, name) {
+  onAllSessionsClosed = onAllSessionsClosedAfterTests(name);
+  startServer();
+  server.hooks.POST.push(unitTestPostHandler);
 
   const startUrl = `http://${host}:${server.port}${testUrl}`;
-  startBrowsers(startUrl, function (session) {
+  startBrowsers(function (session) {
+    session.numRuns = 0;
+    session.numErrors = 0;
+  }, makeTestUrl(startUrl));
+}
+
+function startIntegrationTest() {
+  onAllSessionsClosed = onAllSessionsClosedAfterTests("integration");
+  startServer();
+
+  const { runTests } = require("./integration-boot.js");
+  startBrowsers(function (session) {
     session.numRuns = 0;
     session.numErrors = 0;
   });
+  global.integrationBaseUrl = `http://${host}:${server.port}/build/generic/web/viewer.html`;
+  global.integrationSessions = sessions;
+
+  Promise.all(sessions.map(session => session.browserPromise)).then(
+    async () => {
+      const results = { runs: 0, failures: 0 };
+      await runTests(results);
+      sessions[0].numRuns = results.runs;
+      sessions[0].numErrors = results.failures;
+      await Promise.all(sessions.map(session => closeSession(session.name)));
+    }
+  );
 }
 
 function unitTestPostHandler(req, res) {
@@ -763,12 +899,11 @@ function unitTestPostHandler(req, res) {
   return true;
 }
 
-async function startBrowser(browserName, startUrl) {
-  const revisions = require("puppeteer/package.json").puppeteer;
+async function startBrowser(browserName, startUrl = "") {
+  const revisions =
+    require("puppeteer/lib/cjs/puppeteer/revisions.js").PUPPETEER_REVISIONS;
   const wantedRevision =
-    browserName === "chrome"
-      ? revisions.chrome_revision
-      : revisions.firefox_revision;
+    browserName === "chrome" ? revisions.chromium : revisions.firefox;
 
   // Remove other revisions than the one we want to use. Updating Puppeteer can
   // cause a new revision to be used, and not removing older revisions causes
@@ -786,22 +921,61 @@ async function startBrowser(browserName, startUrl) {
     }
   }
 
-  const browser = await puppeteer.launch({
+  const options = {
     product: browserName,
     headless: false,
     defaultViewport: null,
-    // Firefox must complete its execution before starting, mainly on Windows.
-    // Refer to https://github.com/puppeteer/puppeteer/issues/5376 and
-    // https://phabricator.services.mozilla.com/D6702.
-    args: browserName === "firefox" ? ["--wait-for-browser"] : [],
-  });
-  const pages = await browser.pages();
-  const page = pages[0];
-  await page.goto(startUrl, { timeout: 0 });
+    ignoreDefaultArgs: ["--disable-extensions"],
+  };
+
+  if (!tempDir) {
+    tempDir = fs.mkdtempSync(path.join(os.tmpdir(), "pdfjs-"));
+  }
+  const printFile = path.join(tempDir, "print.pdf");
+
+  if (browserName === "chrome") {
+    // avoid crash
+    options.args = ["--no-sandbox", "--disable-setuid-sandbox"];
+    // silent printing in a pdf
+    options.args.push("--kiosk-printing");
+  }
+
+  if (browserName === "firefox") {
+    options.extraPrefsFirefox = {
+      // avoid to have a prompt when leaving a page with a form
+      "dom.disable_beforeunload": true,
+      // Disable dialog when saving a pdf
+      "pdfjs.disabled": true,
+      "browser.helperApps.neverAsk.saveToDisk": "application/pdf",
+      // Avoid popup when saving is done
+      "browser.download.always_ask_before_handling_new_types": true,
+      "browser.download.panel.shown": true,
+      // Save file in output
+      "browser.download.folderList": 2,
+      "browser.download.dir": tempDir,
+      // Print silently in a pdf
+      "print.always_print_silent": true,
+      "print.show_print_progress": false,
+      print_printer: "PDF",
+      "print.printer_PDF.print_to_file": true,
+      "print.printer_PDF.print_to_filename": printFile,
+      // Enable OffscreenCanvas
+      "gfx.offscreencanvas.enabled": true,
+    };
+  }
+
+  const browser = await puppeteer.launch(options);
+
+  if (startUrl) {
+    const pages = await browser.pages();
+    const page = pages[0];
+    await page.goto(startUrl, { timeout: 0, waitUntil: "domcontentloaded" });
+  }
+
   return browser;
 }
 
-function startBrowsers(rootUrl, initSessionCallback) {
+function startBrowsers(initSessionCallback, makeStartUrl = null) {
   const browserNames = options.noChrome ? ["firefox"] : ["firefox", "chrome"];
 
   sessions = [];
@@ -820,16 +994,9 @@ function startBrowsers(rootUrl, initSessionCallback) {
       closed: false,
     };
     sessions.push(session);
+    const startUrl = makeStartUrl ? makeStartUrl(browserName) : "";
 
-    const queryParameters =
-      `?browser=${encodeURIComponent(browserName)}` +
-      `&manifestFile=${encodeURIComponent("/test/" + options.manifestFile)}` +
-      `&testFilter=${JSON.stringify(options.testfilter)}` +
-      `&delay=${options.statsDelay}` +
-      `&masterMode=${options.masterMode}`;
-    const startUrl = rootUrl + queryParameters;
-
-    startBrowser(browserName, startUrl)
+    session.browserPromise = startBrowser(browserName, startUrl)
       .then(function (browser) {
         session.browser = browser;
         if (initSessionCallback) {
@@ -837,7 +1004,7 @@ function startBrowsers(rootUrl, initSessionCallback) {
         }
       })
       .catch(function (ex) {
-        console.log(`Error while starting ${browserName}: ${ex}`);
+        console.log(`Error while starting ${browserName}: ${ex.message}`);
         closeSession(browserName);
       });
   }
@@ -877,8 +1044,15 @@ async function closeSession(browser) {
     const allClosed = sessions.every(function (s) {
       return s.closed;
     });
-    if (allClosed && onAllSessionsClosed) {
-      onAllSessionsClosed();
+    if (allClosed) {
+      if (tempDir) {
+        const rimraf = require("rimraf");
+        rimraf.sync(tempDir);
+      }
+
+      if (onAllSessionsClosed) {
+        onAllSessionsClosed();
+      }
     }
   }
 }
@@ -920,6 +1094,11 @@ function main() {
     });
   } else if (options.fontTest) {
     startUnitTest("/test/font/font_test.html", "font");
+  } else if (options.integration) {
+    // Allows linked PDF files in integration-tests as well.
+    ensurePDFsDownloaded(function () {
+      startIntegrationTest();
+    });
   } else {
     startRefTest(options.masterMode, options.reftest);
   }
@@ -931,5 +1110,6 @@ var onAllSessionsClosed;
 var host = "127.0.0.1";
 var options = parseOptions();
 var stats;
+var tempDir = null;
 
 main();
