@@ -18,6 +18,9 @@ import { isNodeJS } from "../../src/shared/is_node.js";
 import { PDFNodeStream } from "../../src/display/node_stream.js";
 import { setPDFNetworkStreamFactory } from "../../src/display/api.js";
 
+// Sets longer timeout, similar to `jasmine-boot.js`.
+jasmine.DEFAULT_TIMEOUT_INTERVAL = 30000;
+
 // Ensure that this script only runs in Node.js environments.
 if (!isNodeJS) {
   throw new Error(
