@@ -16,14 +16,10 @@
 import {
   Cmd,
   Dict,
-  EOF,
   isCmd,
   isDict,
-  isEOF,
   isName,
-  isRef,
   isRefsEqual,
-  isStream,
   Name,
   Ref,
   RefSet,
@@ -50,6 +46,21 @@ describe("primitives", function () {
       expect(firstSubtype).toBe(secondSubtype);
       expect(firstFont).not.toBe(firstSubtype);
     });
+
+    it("should create only one object for *empty* names and cache it", function () {
+      const firstEmpty = Name.get("");
+      const secondEmpty = Name.get("");
+      const normalName = Name.get("string");
+
+      expect(firstEmpty).toBe(secondEmpty);
+      expect(firstEmpty).not.toBe(normalName);
+    });
+
+    it("should not accept to create a non-string name", function () {
+      expect(function () {
+        Name.get(123);
+      }).toThrow(new Error('Name: The "name" must be a string.'));
+    });
   });
 
   describe("Cmd", function () {
@@ -69,6 +80,12 @@ describe("primitives", function () {
       expect(firstET).toBe(secondET);
       expect(firstBT).not.toBe(firstET);
     });
+
+    it("should not accept to create a non-string cmd", function () {
+      expect(function () {
+        Cmd.get(123);
+      }).toThrow(new Error('Cmd: The "cmd" must be a string.'));
+    });
   });
 
   describe("Dict", function () {
@@ -80,7 +97,7 @@ describe("primitives", function () {
     const checkInvalidKeyValues = function (dict) {
       expect(dict.get()).toBeUndefined();
       expect(dict.get("Prev")).toBeUndefined();
-      expect(dict.get("Decode", "D")).toBeUndefined();
+      expect(dict.get("D", "Decode")).toBeUndefined();
       expect(dict.get("FontFile", "FontFile2", "FontFile3")).toBeUndefined();
     };
 
@@ -90,7 +107,7 @@ describe("primitives", function () {
     const testFontFile2 = "file2";
     const testFontFile3 = "file3";
 
-    beforeAll(function (done) {
+    beforeAll(function () {
       emptyDict = new Dict();
 
       dictWithSizeKey = new Dict();
@@ -100,8 +117,6 @@ describe("primitives", function () {
       dictWithManyKeys.set("FontFile", testFontFile);
       dictWithManyKeys.set("FontFile2", testFontFile2);
       dictWithManyKeys.set("FontFile3", testFontFile3);
-
-      done();
     });
 
     afterAll(function () {
@@ -146,6 +161,17 @@ describe("primitives", function () {
       checkInvalidKeyValues(dictWithSizeKey);
     });
 
+    it("should not accept to set a non-string key", function () {
+      const dict = new Dict();
+      expect(function () {
+        dict.set(123, "val");
+      }).toThrow(new Error('Dict.set: The "key" must be a string.'));
+
+      expect(dict.has(123)).toBeFalsy();
+
+      checkInvalidKeyValues(dict);
+    });
+
     it("should not accept to set a key with an undefined value", function () {
       const dict = new Dict();
       expect(function () {
@@ -171,40 +197,28 @@ describe("primitives", function () {
       ).toEqual(testFontFile);
     });
 
-    it("should asynchronously fetch unknown keys", function (done) {
+    it("should asynchronously fetch unknown keys", async function () {
       const keyPromises = [
         dictWithManyKeys.getAsync("Size"),
         dictWithSizeKey.getAsync("FontFile", "FontFile2", "FontFile3"),
       ];
 
-      Promise.all(keyPromises)
-        .then(function (values) {
-          expect(values[0]).toBeUndefined();
-          expect(values[1]).toBeUndefined();
-          done();
-        })
-        .catch(function (reason) {
-          done.fail(reason);
-        });
+      const values = await Promise.all(keyPromises);
+      expect(values[0]).toBeUndefined();
+      expect(values[1]).toBeUndefined();
     });
 
-    it("should asynchronously fetch correct values for multiple stored keys", function (done) {
+    it("should asynchronously fetch correct values for multiple stored keys", async function () {
       const keyPromises = [
         dictWithManyKeys.getAsync("FontFile3"),
         dictWithManyKeys.getAsync("FontFile2", "FontFile3"),
         dictWithManyKeys.getAsync("FontFile", "FontFile2", "FontFile3"),
       ];
 
-      Promise.all(keyPromises)
-        .then(function (values) {
-          expect(values[0]).toEqual(testFontFile3);
-          expect(values[1]).toEqual(testFontFile2);
-          expect(values[2]).toEqual(testFontFile);
-          done();
-        })
-        .catch(function (reason) {
-          done.fail(reason);
-        });
+      const values = await Promise.all(keyPromises);
+      expect(values[0]).toEqual(testFontFile3);
+      expect(values[1]).toEqual(testFontFile2);
+      expect(values[2]).toEqual(testFontFile);
     });
 
     it("should callback for each stored key", function () {
@@ -220,7 +234,7 @@ describe("primitives", function () {
       expect(callbackSpyCalls.count()).toEqual(3);
     });
 
-    it("should handle keys pointing to indirect objects, both sync and async", function (done) {
+    it("should handle keys pointing to indirect objects, both sync and async", async function () {
       const fontRef = Ref.get(1, 0);
       const xref = new XRefMock([{ ref: fontRef, data: testFontFile }]);
       const fontDict = new Dict(xref);
@@ -231,15 +245,12 @@ describe("primitives", function () {
         testFontFile
       );
 
-      fontDict
-        .getAsync("FontFile", "FontFile2", "FontFile3")
-        .then(function (value) {
-          expect(value).toEqual(testFontFile);
-          done();
-        })
-        .catch(function (reason) {
-          done.fail(reason);
-        });
+      const value = await fontDict.getAsync(
+        "FontFile",
+        "FontFile2",
+        "FontFile3"
+      );
+      expect(value).toEqual(testFontFile);
     });
 
     it("should handle arrays containing indirect objects", function () {
@@ -407,33 +418,45 @@ describe("primitives", function () {
   });
 
   describe("RefSet", function () {
-    it("should have a stored value", function () {
-      const ref = Ref.get(4, 2);
-      const refset = new RefSet();
-      refset.put(ref);
-      expect(refset.has(ref)).toBeTruthy();
-    });
-    it("should not have an unknown value", function () {
-      const ref = Ref.get(4, 2);
-      const refset = new RefSet();
-      expect(refset.has(ref)).toBeFalsy();
+    const ref1 = Ref.get(4, 2),
+      ref2 = Ref.get(5, 2);
+    let refSet;
 
-      refset.put(ref);
-      const anotherRef = Ref.get(2, 4);
-      expect(refset.has(anotherRef)).toBeFalsy();
+    beforeEach(function () {
+      refSet = new RefSet();
+    });
+
+    afterEach(function () {
+      refSet = null;
+    });
+
+    it("should have a stored value", function () {
+      refSet.put(ref1);
+      expect(refSet.has(ref1)).toBeTruthy();
+    });
+
+    it("should not have an unknown value", function () {
+      expect(refSet.has(ref1)).toBeFalsy();
+      refSet.put(ref1);
+      expect(refSet.has(ref2)).toBeFalsy();
+    });
+
+    it("should support iteration", function () {
+      refSet.put(ref1);
+      refSet.put(ref2);
+      expect([...refSet]).toEqual([ref1.toString(), ref2.toString()]);
     });
   });
 
   describe("RefSetCache", function () {
-    const ref1 = Ref.get(4, 2);
-    const ref2 = Ref.get(5, 2);
-    const obj1 = Name.get("foo");
-    const obj2 = Name.get("bar");
+    const ref1 = Ref.get(4, 2),
+      ref2 = Ref.get(5, 2),
+      obj1 = Name.get("foo"),
+      obj2 = Name.get("bar");
     let cache;
 
-    beforeEach(function (done) {
+    beforeEach(function () {
       cache = new RefSetCache();
-      done();
     });
 
     afterEach(function () {
@@ -473,27 +496,13 @@ describe("primitives", function () {
     it("should support iteration", function () {
       cache.put(ref1, obj1);
       cache.put(ref2, obj2);
-
-      const values = [];
-      cache.forEach(function (value) {
-        values.push(value);
-      });
-      expect(values).toEqual([obj1, obj2]);
-    });
-  });
-
-  describe("isEOF", function () {
-    it("handles non-EOF", function () {
-      const nonEOF = "foo";
-      expect(isEOF(nonEOF)).toEqual(false);
-    });
-
-    it("handles EOF", function () {
-      expect(isEOF(EOF)).toEqual(true);
+      expect([...cache]).toEqual([obj1, obj2]);
     });
   });
 
   describe("isName", function () {
+    /* eslint-disable no-restricted-syntax */
+
     it("handles non-names", function () {
       const nonName = {};
       expect(isName(nonName)).toEqual(false);
@@ -509,9 +518,21 @@ describe("primitives", function () {
       expect(isName(name, "Font")).toEqual(true);
       expect(isName(name, "Subtype")).toEqual(false);
     });
+
+    it("handles *empty* names, with name check", function () {
+      const emptyName = Name.get("");
+
+      expect(isName(emptyName)).toEqual(true);
+      expect(isName(emptyName, "")).toEqual(true);
+      expect(isName(emptyName, "string")).toEqual(false);
+    });
+
+    /* eslint-enable no-restricted-syntax */
   });
 
   describe("isCmd", function () {
+    /* eslint-disable no-restricted-syntax */
+
     it("handles non-commands", function () {
       const nonCmd = {};
       expect(isCmd(nonCmd)).toEqual(false);
@@ -527,9 +548,13 @@ describe("primitives", function () {
       expect(isCmd(cmd, "BT")).toEqual(true);
       expect(isCmd(cmd, "ET")).toEqual(false);
     });
+
+    /* eslint-enable no-restricted-syntax */
   });
 
   describe("isDict", function () {
+    /* eslint-disable no-restricted-syntax */
+
     it("handles non-dictionaries", function () {
       const nonDict = {};
       expect(isDict(nonDict)).toEqual(false);
@@ -547,18 +572,8 @@ describe("primitives", function () {
       expect(isDict(dict, "Page")).toEqual(true);
       expect(isDict(dict, "Contents")).toEqual(false);
     });
-  });
 
-  describe("isRef", function () {
-    it("handles non-refs", function () {
-      const nonRef = {};
-      expect(isRef(nonRef)).toEqual(false);
-    });
-
-    it("handles refs", function () {
-      const ref = Ref.get(1, 0);
-      expect(isRef(ref)).toEqual(true);
-    });
+    /* eslint-enable no-restricted-syntax */
   });
 
   describe("isRefsEqual", function () {
@@ -574,16 +589,4 @@ describe("primitives", function () {
       expect(isRefsEqual(ref1, ref2)).toEqual(false);
     });
   });
-
-  describe("isStream", function () {
-    it("handles non-streams", function () {
-      const nonStream = {};
-      expect(isStream(nonStream)).toEqual(false);
-    });
-
-    it("handles streams", function () {
-      const stream = new StringStream("foo");
-      expect(isStream(stream)).toEqual(true);
-    });
-  });
 });
